@@ -34,6 +34,7 @@
 #  include <config.h>
 #endif
 
+#include <stdint.h>
 #include "pycairo-private.h"
 
 
@@ -48,19 +49,16 @@
  * Create a new PycairoSurface from a cairo_surface_t
  * surface - a cairo_surface_t to 'wrap' into a Python object.
  *           it is unreferenced if the PycairoSurface creation fails, or if
-             the cairo_surface_t has an error status
- * type - the type of the object to instantiate; it can be NULL,
- *        meaning a base cairo.Surface type, or it can be a subclass of
- *        cairo.Surface.
+ *           the cairo_surface_t has an error status
  * base - the base object used to create the context, or NULL.
  *        it is referenced to keep it alive while the cairo_surface_t is
  *        being used
  * Return value: New reference or NULL on failure
  */
 PyObject *
-PycairoSurface_FromSurface (cairo_surface_t *surface, PyTypeObject *type,
-			    PyObject *base)
+PycairoSurface_FromSurface (cairo_surface_t *surface, PyObject *base)
 {
+    PyTypeObject *type;
     PyObject *o;
 
     assert (surface != NULL);
@@ -70,19 +68,70 @@ PycairoSurface_FromSurface (cairo_surface_t *surface, PyTypeObject *type,
 	return NULL;
     }
 
-    if (type == NULL)
-        type = &PycairoSurface_Type;
-    o = PycairoSurface_Type.tp_alloc (type, 0);
-    if (o) {
+    switch (cairo_surface_get_type (surface)) {
+    case CAIRO_SURFACE_TYPE_IMAGE:
+	type = &PycairoImageSurface_Type;
+	break;
+#if CAIRO_HAS_PDF_SURFACE
+    case CAIRO_SURFACE_TYPE_PDF:
+	type = &PycairoPDFSurface_Type;
+	break;
+#endif
+#if CAIRO_HAS_PS_SURFACE
+    case CAIRO_SURFACE_TYPE_PS:
+	type = &PycairoPSSurface_Type;
+	break;
+#endif
+#if CAIRO_HAS_SVG_SURFACE
+    case CAIRO_SURFACE_TYPE_SVG:
+	type = &PycairoSVGSurface_Type;
+	break;
+#endif
+#if CAIRO_HAS_WIN32_SURFACE
+    case CAIRO_SURFACE_TYPE_WIN32:
+	type = &PycairoWin32Surface_Type;
+	break;
+#endif
+#if CAIRO_HAS_XLIB_SURFACE
+    case CAIRO_SURFACE_TYPE_XLIB:
+	type = &PycairoXlibSurface_Type;
+	break;
+#endif
+    default:
+	PyErr_SetString(CairoError, "Unsupported Surface type");
+	return NULL;
+    }
+
+    o = type->tp_alloc (type, 0);
+    if (o == NULL) {
+	cairo_surface_destroy (surface);
+    } else {
 	((PycairoSurface *)o)->surface = surface;
 	Py_XINCREF(base);
 	((PycairoSurface *)o)->base = base;
-    } else {
-	cairo_surface_destroy (surface);
     }
     return o;
 }
 
+/* for use with
+ * cairo_surface_write_to_png_stream()
+ * cairo_pdf/ps/svg_surface_create_for_stream()
+ */
+static cairo_status_t
+_write_func (void *closure, const unsigned char *data, unsigned int length)
+{
+    PyObject *res = PyObject_CallMethod ((PyObject *)closure, "write", "(s#)",
+					 data, length);
+    if (res == NULL) {
+	/* an exception has occurred, it will be picked up later by
+	 * Pycairo_Check_Status()
+	 */
+	return CAIRO_STATUS_WRITE_ERROR;
+    }
+    Py_DECREF(res);
+    return CAIRO_STATUS_SUCCESS;
+}
+
 static void
 surface_dealloc (PycairoSurface *o)
 {
@@ -112,20 +161,15 @@ surface_new (PyTypeObject *type, PyObject *args, PyObject *kwds)
 static PyObject *
 surface_create_similar (PycairoSurface *o, PyObject *args)
 {
-    cairo_surface_t *surface;
     cairo_content_t content;
     int width, height;
 
     if (!PyArg_ParseTuple (args, "iii:Surface.create_similar",
 			   &content, &width, &height))
 	return NULL;
-
-    surface = cairo_surface_create_similar (o->surface, content,
-					    width, height);
-    /* bug #2765 - "How do we identify surface types?"
-     * should pass surface type as arg2
-     */
-    return PycairoSurface_FromSurface (surface, NULL, NULL);
+    return PycairoSurface_FromSurface (
+	     cairo_surface_create_similar (o->surface, content, width, height),
+	     NULL);
 }
 
 static PyObject *
@@ -149,6 +193,20 @@ surface_flush (PycairoSurface *o)
     Py_RETURN_NONE;
 }
 
+static PyObject *
+surface_get_content (PycairoSurface *o)
+{
+    return PyInt_FromLong (cairo_surface_get_content (o->surface));
+}
+
+static PyObject *
+surface_get_device_offset (PycairoSurface *o)
+{
+    double x_offset, y_offset;
+    cairo_surface_get_device_offset (o->surface, &x_offset, &y_offset);
+    return Py_BuildValue("(dd)", x_offset, y_offset);
+}
+
 static PyObject *
 surface_get_font_options (PycairoSurface *o)
 {
@@ -190,42 +248,43 @@ surface_set_device_offset (PycairoSurface *o, PyObject *args)
     Py_RETURN_NONE;
 }
 
-#ifdef CAIRO_HAS_PNG_FUNCTIONS
-static cairo_status_t
-_write_func (void *closure, const unsigned char *data,
-	     unsigned int length)
+static PyObject *
+surface_set_fallback_resolution (PycairoSurface *o, PyObject *args)
 {
-    if (fwrite (data, 1, (size_t) length, (FILE *)closure) != length)
-	return CAIRO_STATUS_WRITE_ERROR;
-    return CAIRO_STATUS_SUCCESS;
+    double x_ppi, y_ppi;
+
+    if (!PyArg_ParseTuple(args, "dd:Surface.set_fallback_resolution",
+			  &x_ppi, &y_ppi))
+	return NULL;
+    cairo_surface_set_fallback_resolution (o->surface, x_ppi, y_ppi);
+    Py_RETURN_NONE;
 }
 
+#ifdef CAIRO_HAS_PNG_FUNCTIONS
 /* METH_O */
 static PyObject *
 surface_write_to_png (PycairoSurface *o, PyObject *file)
 {
-    FILE *fp;
     cairo_status_t status;
 
     if (PyObject_TypeCheck (file, &PyBaseString_Type)) {
-	fp = fopen (PyString_AsString(file), "wb");
-	if (fp == NULL) {
-	    PyErr_SetString(PyExc_IOError, "unable to open file for writing");
+	/* string (filename) argument */
+	status = cairo_surface_write_to_png (o->surface,
+					     PyString_AsString(file));
+
+    } else {  /* file or file-like object argument */
+	PyObject* writer = PyObject_GetAttrString (file, "write");
+	if (writer == NULL || !PyCallable_Check (writer)) {
+	    Py_XDECREF(writer);
+	    PyErr_SetString(PyExc_TypeError,
+"Surface.write_to_png takes one argument which must be a filename (str), file "
+"object, or a file-like object which has a \"write\" method (like StringIO)");
 	    return NULL;
 	}
-    } else if (PyObject_TypeCheck (file, &PyFile_Type)) {
-	fp = PyFile_AsFile(file);
-
-    } else {
-	PyErr_SetString(PyExc_TypeError,
-			"Surface.write_to_png takes one argument "
-			"which must be a filename (str) or file object");
-	return NULL;
+	Py_DECREF(writer);
+	status = cairo_surface_write_to_png_stream (o->surface, _write_func,
+						    file);
     }
-    status = cairo_surface_write_to_png_stream (o->surface, _write_func, fp);
-    if (PyObject_TypeCheck (file, &PyBaseString_Type))
-    	fclose (fp);
-
     if (Pycairo_Check_Status (status))
 	return NULL;
     Py_RETURN_NONE;
@@ -236,17 +295,23 @@ surface_write_to_png (PycairoSurface *o, PyObject *file)
 static PyMethodDef surface_methods[] = {
     /* methods never exposed in a language binding:
      * cairo_surface_destroy()
-     * cairo_surface_reference()
+     * cairo_surface_get_type()
      * cairo_surface_get_user_data()
+     * cairo_surface_reference()
      * cairo_surface_set_user_data()
      */
     {"create_similar", (PyCFunction)surface_create_similar,    METH_VARARGS },
     {"finish",         (PyCFunction)surface_finish,            METH_NOARGS },
     {"flush",          (PyCFunction)surface_flush,             METH_NOARGS },
+    {"get_content",    (PyCFunction)surface_get_content,       METH_NOARGS },
+    {"get_device_offset",(PyCFunction)surface_get_device_offset,
+                                                                METH_NOARGS },
     {"get_font_options",(PyCFunction)surface_get_font_options, METH_NOARGS },
     {"mark_dirty",     (PyCFunction)surface_mark_dirty,        METH_KEYWORDS },
     {"set_device_offset",(PyCFunction)surface_set_device_offset,
                                                                 METH_VARARGS },
+    {"set_fallback_resolution",(PyCFunction)surface_set_fallback_resolution,
+                                                                METH_VARARGS },
 #ifdef CAIRO_HAS_PNG_FUNCTIONS
     {"write_to_png",   (PyCFunction)surface_write_to_png,      METH_O },
 #endif
@@ -307,24 +372,13 @@ image_surface_new (PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
     cairo_format_t format;
     int width, height;
-    cairo_surface_t *surface;
-    PyObject *o;
 
     if (!PyArg_ParseTuple (args, "iii:ImageSurface.__new__",
 			   &format, &width, &height))
 	return NULL;
-
-    o = type->tp_alloc(type, 0);
-    if (o) {
-	surface = cairo_image_surface_create (format, width, height);
-	if (Pycairo_Check_Status (cairo_surface_status (surface))) {
-	    cairo_surface_destroy (surface);
-	    Py_DECREF(o);
-	    return NULL;
-	}
-	((PycairoImageSurface *)o)->surface = surface;
-    }
-    return o;
+    return PycairoSurface_FromSurface (
+	       cairo_image_surface_create (format, width, height),
+	       NULL);
 }
 
 #ifdef HAVE_NUMPY
@@ -383,8 +437,7 @@ image_surface_create_for_array (PyTypeObject *type, PyObject *args)
 					  array->dimensions[1],
 					  array->dimensions[0],
 					  array->strides[0]);
-    return PycairoSurface_FromSurface(surface, &PycairoImageSurface_Type,
-				      (PyObject *)array);
+    return PycairoSurface_FromSurface(surface, (PyObject *)array);
 }
 #endif /* HAVE_NUMPY */
 
@@ -417,17 +470,21 @@ image_surface_create_for_data (PyTypeObject *type, PyObject *args)
     if (stride < 0) {
 	switch (format) {
 	case CAIRO_FORMAT_ARGB32:
+	case CAIRO_FORMAT_RGB24:
 	    stride = width * 4;
 	    break;
-	case CAIRO_FORMAT_RGB24:
-	    stride = width * 3;
+	case CAIRO_FORMAT_RGB16_565:
+	    stride = width * 2;
 	    break;
 	case CAIRO_FORMAT_A8:
 	    stride = width;
 	    break;
-	case CAIRO_FORMAT_A1:
+	case CAIRO_FORMAT_A1: /* should be stride = (width + 31) / 32 * 4 ? */
 	    stride = (width + 1) / 8;
 	    break;
+	default:
+	    PyErr_SetString(CairoError, "Unknown format");
+	    return NULL;
 	}
     }
     if (height * stride > buffer_len) {
@@ -436,7 +493,7 @@ image_surface_create_for_data (PyTypeObject *type, PyObject *args)
     }
     surface = cairo_image_surface_create_for_data (buffer, format, width,
 						   height, stride);
-    return PycairoSurface_FromSurface(surface, &PycairoImageSurface_Type, obj);
+    return PycairoSurface_FromSurface(surface, obj);
 }
 
 
@@ -444,56 +501,128 @@ image_surface_create_for_data (PyTypeObject *type, PyObject *args)
 static cairo_status_t
 _read_func (void *closure, unsigned char *data, unsigned int length)
 {
-    if (fread (data, 1, (size_t) length, (FILE *)closure) != length)
+    char *str;
+    PyObject *pystr = PyObject_CallMethod ((PyObject *)closure, "read", "(i)",
+					   length);
+    if (pystr == NULL)
 	return CAIRO_STATUS_READ_ERROR;
+    str = PyString_AsString(pystr);
+    Py_DECREF(pystr);
+    if (str == NULL)
+	return CAIRO_STATUS_READ_ERROR;
+    /* don't use strncpy() since png data may contain NUL bytes */
+    memcpy (data, str, length);
     return CAIRO_STATUS_SUCCESS;
 }
 
 /* METH_O | METH_CLASS */
 static PyObject *
-image_surface_create_from_png (PyTypeObject *type, PyObject *o)
+image_surface_create_from_png (PyTypeObject *type, PyObject *file)
 {
-    FILE *fp;
-    cairo_surface_t *surface;
-
-    if (PyObject_TypeCheck (o, &PyBaseString_Type)) {
-	fp = fopen (PyString_AsString(o), "rb");
-	if (fp == NULL) {
-	    PyErr_SetString(PyExc_IOError, "unable to open file for reading");
-	    return NULL;
-	}
-
-    } else if (PyObject_TypeCheck (o, &PyFile_Type)) {
-	fp = PyFile_AsFile(o);
-
-    } else {
-	PyErr_SetString(PyExc_TypeError,
-			"ImageSurface.create_from_png takes one argument "
-			"which must be a filename (str) or file object");
-	return NULL;
+    if (PyObject_TypeCheck (file, &PyBaseString_Type)) {
+	return PycairoSurface_FromSurface (
+            cairo_image_surface_create_from_png (PyString_AsString(file)),
+            NULL);
     }
 
-    surface = cairo_image_surface_create_from_png_stream (_read_func, fp);
-    if (PyObject_TypeCheck (o, &PyBaseString_Type))
-	fclose (fp);
+    /* file or file-like object argument */
+    PyObject* reader = PyObject_GetAttrString (file, "read");
+    if (reader == NULL || !PyCallable_Check (reader)) {
+	Py_XDECREF(reader);
+	PyErr_SetString(PyExc_TypeError,
+"ImageSurface.create_from_png argument must be a filename (str), file object, "
+"or an object that has a \"read\" method (like StringIO)");
+	return NULL;
+    }
+    Py_DECREF(reader);
 
-    return PycairoSurface_FromSurface (surface, &PycairoImageSurface_Type,
-				       NULL);
+    return PycairoSurface_FromSurface (
+        cairo_image_surface_create_from_png_stream (_read_func, file), NULL);
 }
 #endif /* CAIRO_HAS_PNG_FUNCTIONS */
 
+static PyObject *
+image_surface_get_format (PycairoImageSurface *o)
+{
+    return PyInt_FromLong (cairo_image_surface_get_format (o->surface));
+}
+
 static PyObject *
 image_surface_get_height (PycairoImageSurface *o)
 {
-    return Py_BuildValue("i", cairo_image_surface_get_height (o->surface));
+    return PyInt_FromLong (cairo_image_surface_get_height (o->surface));
 }
 
 static PyObject *
 image_surface_get_width (PycairoImageSurface *o)
 {
-    return Py_BuildValue("i", cairo_image_surface_get_width (o->surface));
+    return PyInt_FromLong (cairo_image_surface_get_width (o->surface));
 }
 
+static PyObject *
+image_surface_get_stride (PycairoImageSurface *o)
+{
+    return PyInt_FromLong (cairo_image_surface_get_stride (o->surface));
+}
+
+/* This was modified from cairo/src/cairo-png.c unpremultiply_data() */
+/* ARGB32 (native-endian, premultiplied) => RGBA */
+static void
+_argb32_to_unpremultiplied_rgba (uint8_t *data, int length)
+{
+    unsigned int i;
+
+    for (i = 0; i < length; i += 4) {
+        uint8_t *b = &data[i];
+        uint32_t pixel;
+        uint8_t  alpha;
+
+	memcpy (&pixel, b, sizeof (uint32_t));
+	alpha = (pixel & 0xff000000) >> 24;
+        if (alpha == 0) {
+	    b[0] = b[1] = b[2] = b[3] = 0;
+	} else {
+            b[0] = (((pixel & 0xff0000) >> 16) * 255 + alpha / 2) / alpha;
+            b[1] = (((pixel & 0x00ff00) >>  8) * 255 + alpha / 2) / alpha;
+            b[2] = (((pixel & 0x0000ff) >>  0) * 255 + alpha / 2) / alpha;
+	    b[3] = alpha;
+	}
+    }
+}
+
+/* return a Python buffer object containing the ImageSurface data in RGBA
+ * format.
+ */
+static PyObject *
+image_surface_get_data_as_rgba (PycairoImageSurface *o)
+{
+    PyObject *buf;
+    unsigned char *data;
+    uint8_t *buffer;
+    int height, stride, length;
+    cairo_surface_t *surface = o->surface;
+    cairo_format_t format = cairo_image_surface_get_format (surface);
+
+    if (format != CAIRO_FORMAT_ARGB32) {
+	PyErr_SetString(PyExc_TypeError, "ImageSurface.to_rgba() can only be "
+			"called on a cairo.FORMAT_ARGB32 surface");
+	return NULL;
+    }
+    data   = cairo_image_surface_get_data (surface);
+    height = cairo_image_surface_get_height (surface);
+    stride = cairo_image_surface_get_stride (surface);
+
+    buf = PyBuffer_New(height * stride);
+    if (buf != NULL) {
+	if (PyObject_AsWriteBuffer(buf, (void **)&buffer, &length)) {
+	    Py_DECREF(buf);
+	    return NULL;
+	}
+	memcpy (buffer, data, length);
+	_argb32_to_unpremultiplied_rgba (buffer, length);
+    }
+    return buf;
+}
 
 static PyMethodDef image_surface_methods[] = {
 #ifdef HAVE_NUMPY
@@ -506,8 +635,12 @@ static PyMethodDef image_surface_methods[] = {
     {"create_from_png", (PyCFunction)image_surface_create_from_png,
                                                    METH_O | METH_CLASS },
 #endif
-    {"get_height",    (PyCFunction)image_surface_get_height,      METH_NOARGS},
-    {"get_width",     (PyCFunction)image_surface_get_width,       METH_NOARGS},
+    {"get_format",    (PyCFunction)image_surface_get_format,     METH_NOARGS},
+    {"get_height",    (PyCFunction)image_surface_get_height,     METH_NOARGS},
+    {"get_width",     (PyCFunction)image_surface_get_width,      METH_NOARGS},
+    {"get_stride",    (PyCFunction)image_surface_get_stride,     METH_NOARGS},
+    {"get_data_as_rgba",(PyCFunction)image_surface_get_data_as_rgba,
+                                                                 METH_NOARGS},
     {NULL, NULL, 0, NULL},
 };
 
@@ -565,42 +698,52 @@ PyTypeObject PycairoImageSurface_Type = {
 static PyObject *
 pdf_surface_new (PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
-    const char *filename;
     double width_in_points, height_in_points;
-    cairo_surface_t *surface;
-    PyObject *o;
+    PyObject *file;
 
-    if (!PyArg_ParseTuple(args, "sdd:PDFSurface.__new__",
-			  &filename, &width_in_points, &height_in_points))
+    if (!PyArg_ParseTuple(args, "Odd:PDFSurface.__new__",
+			  &file, &width_in_points, &height_in_points))
 	return NULL;
 
-    o = type->tp_alloc(type, 0);
-    if (o) {
-	surface = cairo_pdf_surface_create (filename, width_in_points,
-					    height_in_points);
-	if (Pycairo_Check_Status (cairo_surface_status (surface))) {
-	    cairo_surface_destroy (surface);
-	    Py_DECREF(o);
-	    return NULL;
-	}
-	((PycairoPDFSurface *)o)->surface = surface;
+    if (PyObject_TypeCheck (file, &PyBaseString_Type)) {
+	/* string (filename) argument */
+	return PycairoSurface_FromSurface (
+                  cairo_pdf_surface_create (PyString_AsString(file),
+                                     width_in_points, height_in_points),
+	       NULL);
     }
-    return o;
+    /* file or file-like object argument */
+    PyObject* writer = PyObject_GetAttrString (file, "write");
+    if (writer == NULL || !PyCallable_Check (writer)) {
+	Py_XDECREF(writer);
+	PyErr_SetString(PyExc_TypeError,
+"PDFSurface argument 1 must be a filename (str), file object, or an object "
+"that has a \"write\" method (like StringIO)");
+	return NULL;
+    }
+    Py_DECREF(writer);
+
+    return PycairoSurface_FromSurface (
+	       cairo_pdf_surface_create_for_stream (_write_func, file,
+		   width_in_points, height_in_points),
+	   file);
 }
 
 static PyObject *
-pdf_surface_set_dpi (PycairoPDFSurface *o, PyObject *args)
+pdf_surface_set_size (PycairoPDFSurface *o, PyObject *args)
 {
-    double x_dpi, y_dpi;
+    double width_in_points, height_in_points;
 
-    if (!PyArg_ParseTuple(args, "dd:PDFSurface.set_dpi", &x_dpi, &y_dpi))
+    if (!PyArg_ParseTuple(args, "dd:PDFSurface.set_size", &width_in_points,
+			  &height_in_points))
 	return NULL;
-    cairo_pdf_surface_set_dpi (o->surface, x_dpi, y_dpi);
+    cairo_pdf_surface_set_size (o->surface, width_in_points,
+				height_in_points);
     Py_RETURN_NONE;
 }
 
-static PyMethodDef pdfsurface_methods[] = {
-    {"set_dpi", (PyCFunction)pdf_surface_set_dpi,    METH_VARARGS },
+static PyMethodDef pdf_surface_methods[] = {
+    {"set_size", (PyCFunction)pdf_surface_set_size,    METH_VARARGS },
     {NULL, NULL, 0, NULL},
 };
 
@@ -633,7 +776,7 @@ PyTypeObject PycairoPDFSurface_Type = {
     0,                                  /* tp_weaklistoffset */
     0,                                  /* tp_iter */
     0,                                  /* tp_iternext */
-    pdfsurface_methods,                 /* tp_methods */
+    pdf_surface_methods,                /* tp_methods */
     0,                                  /* tp_members */
     0,                                  /* tp_getset */
     0, /* &PycairoSurface_Type, */      /* tp_base */
@@ -658,42 +801,88 @@ PyTypeObject PycairoPDFSurface_Type = {
 static PyObject *
 ps_surface_new (PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
-    const char *filename;
     double width_in_points, height_in_points;
-    cairo_surface_t *surface;
-    PyObject *o;
+    PyObject *file;
 
-    if (!PyArg_ParseTuple(args, "sdd:PSSurface.__new__",
-			  &filename, &width_in_points, &height_in_points))
+    if (!PyArg_ParseTuple(args, "Odd:PSSurface.__new__",
+			  &file, &width_in_points, &height_in_points))
 	return NULL;
 
-    o = type->tp_alloc(type, 0);
-    if (o) {
-	surface = cairo_ps_surface_create (filename, width_in_points,
-					   height_in_points);
-	if (Pycairo_Check_Status (cairo_surface_status (surface))) {
-	    cairo_surface_destroy (surface);
-	    Py_DECREF(o);
-	    return NULL;
-	}
-	((PycairoPSSurface *)o)->surface = surface;
+    if (PyObject_TypeCheck (file, &PyBaseString_Type)) {
+	/* string (filename) argument */
+	return PycairoSurface_FromSurface (
+                  cairo_ps_surface_create (PyString_AsString(file),
+                                     width_in_points, height_in_points),
+	       NULL);
+
     }
-    return o;
+    /* else: file or file-like object argument */
+    PyObject* writer = PyObject_GetAttrString (file, "write");
+    if (writer == NULL || !PyCallable_Check (writer)) {
+	Py_XDECREF(writer);
+	PyErr_SetString(PyExc_TypeError,
+"PSSurface argument 1 must be a filename (str), file object, or an object "
+"that has a \"write\" method (like StringIO)");
+	return NULL;
+    }
+    Py_DECREF(writer);
+
+    return PycairoSurface_FromSurface (
+	       cairo_ps_surface_create_for_stream (_write_func, file,
+                   width_in_points, height_in_points),
+	       file);
 }
 
 static PyObject *
-ps_surface_set_dpi (PycairoPSSurface *o, PyObject *args)
+ps_surface_dsc_begin_page_setup (PycairoPSSurface *o)
 {
-    double x_dpi, y_dpi;
-
-    if (!PyArg_ParseTuple(args, "dd:PSSurface.set_dpi", &x_dpi, &y_dpi))
+    cairo_ps_surface_dsc_begin_page_setup (o->surface);
+    if (Pycairo_Check_Status (cairo_surface_status (o->surface)))
 	return NULL;
-    cairo_ps_surface_set_dpi (o->surface, x_dpi, y_dpi);
     Py_RETURN_NONE;
 }
 
-static PyMethodDef pssurface_methods[] = {
-    {"set_dpi", (PyCFunction)ps_surface_set_dpi,    METH_VARARGS },
+static PyObject *
+ps_surface_dsc_begin_setup (PycairoPSSurface *o)
+{
+    cairo_ps_surface_dsc_begin_setup (o->surface);
+    if (Pycairo_Check_Status (cairo_surface_status (o->surface)))
+	return NULL;
+    Py_RETURN_NONE;
+}
+
+static PyObject *
+ps_surface_dsc_comment (PycairoPSSurface *o, PyObject *args)
+{
+    const char *comment;
+
+    if (!PyArg_ParseTuple(args, "s:PSSurface.dsc_comment", &comment))
+	return NULL;
+
+    cairo_ps_surface_dsc_comment (o->surface, comment);
+    if (Pycairo_Check_Status (cairo_surface_status (o->surface)))
+	return NULL;
+    Py_RETURN_NONE;
+}
+
+static PyObject *
+ps_surface_set_size (PycairoPSSurface *o, PyObject *args)
+{
+    double width_in_points, height_in_points;
+
+    if (!PyArg_ParseTuple(args, "dd:PSSurface.set_size",
+			  &width_in_points, &height_in_points))
+	return NULL;
+    cairo_ps_surface_set_size (o->surface, width_in_points, height_in_points);
+    Py_RETURN_NONE;
+}
+
+static PyMethodDef ps_surface_methods[] = {
+    {"dsc_begin_page_setup",
+                   (PyCFunction)ps_surface_dsc_begin_page_setup, METH_NOARGS },
+    {"dsc_begin_setup", (PyCFunction)ps_surface_dsc_begin_setup, METH_NOARGS },
+    {"dsc_comment", (PyCFunction)ps_surface_dsc_comment,        METH_VARARGS },
+    {"set_size", (PyCFunction)ps_surface_set_size,              METH_VARARGS },
     {NULL, NULL, 0, NULL},
 };
 
@@ -726,7 +915,7 @@ PyTypeObject PycairoPSSurface_Type = {
     0,                                  /* tp_weaklistoffset */
     0,                                  /* tp_iter */
     0,                                  /* tp_iternext */
-    pssurface_methods,                  /* tp_methods */
+    ps_surface_methods,                 /* tp_methods */
     0,                                  /* tp_members */
     0,                                  /* tp_getset */
     0, /* &PycairoSurface_Type, */      /* tp_base */
@@ -744,6 +933,100 @@ PyTypeObject PycairoPSSurface_Type = {
 #endif  /* CAIRO_HAS_PS_SURFACE */
 
 
+/* Class SVGSurface(Surface) ----------------------------------------------- */
+#ifdef CAIRO_HAS_SVG_SURFACE
+#include <cairo-svg.h>
+
+static PyObject *
+svg_surface_new (PyTypeObject *type, PyObject *args, PyObject *kwds)
+{
+    double width_in_points, height_in_points;
+    PyObject *file;
+
+    if (!PyArg_ParseTuple(args, "Odd:SVGSurface.__new__",
+			  &file, &width_in_points, &height_in_points))
+	return NULL;
+
+    if (PyObject_TypeCheck (file, &PyBaseString_Type)) {
+	/* string (filename) argument */
+	return PycairoSurface_FromSurface (
+                  cairo_svg_surface_create (PyString_AsString(file),
+                                     width_in_points, height_in_points),
+	       NULL);
+    }
+    /* else: file or file-like object argument */
+    PyObject* writer = PyObject_GetAttrString (file, "write");
+    if (writer == NULL || !PyCallable_Check (writer)) {
+	Py_XDECREF(writer);
+	PyErr_SetString(PyExc_TypeError,
+"SVGSurface argument 1 must be a filename (str), file object, or an object "
+"that has a \"write\" method (like StringIO)");
+	return NULL;
+    }
+    Py_DECREF(writer);
+
+    return PycairoSurface_FromSurface (
+	       cairo_svg_surface_create_for_stream (_write_func, file,
+	           width_in_points, height_in_points),
+	       file);
+}
+
+static PyMethodDef svg_surface_methods[] = {
+    /* TODO
+     * cairo_svg_surface_restrict_to_version
+     * cairo_svg_get_versions
+     * cairo_svg_version_to_string
+     */
+    {NULL, NULL, 0, NULL},
+};
+
+PyTypeObject PycairoSVGSurface_Type = {
+    PyObject_HEAD_INIT(NULL)
+    0,                                  /* ob_size */
+    "cairo.SVGSurface",                 /* tp_name */
+    sizeof(PycairoSVGSurface),          /* tp_basicsize */
+    0,                                  /* tp_itemsize */
+    0,                                  /* tp_dealloc */
+    0,                                  /* tp_print */
+    0,                                  /* tp_getattr */
+    0,                                  /* tp_setattr */
+    0,                                  /* tp_compare */
+    0,                                  /* tp_repr */
+    0,                                  /* tp_as_number */
+    0,                                  /* tp_as_sequence */
+    0,                                  /* tp_as_mapping */
+    0,                                  /* tp_hash */
+    0,                                  /* tp_call */
+    0,                                  /* tp_str */
+    0,                                  /* tp_getattro */
+    0,                                  /* tp_setattro */
+    0,                                  /* tp_as_buffer */
+    Py_TPFLAGS_DEFAULT,                 /* tp_flags */
+    0,                                  /* tp_doc */
+    0,                                  /* tp_traverse */
+    0,                                  /* tp_clear */
+    0,                                  /* tp_richcompare */
+    0,                                  /* tp_weaklistoffset */
+    0,                                  /* tp_iter */
+    0,                                  /* tp_iternext */
+    svg_surface_methods,                /* tp_methods */
+    0,                                  /* tp_members */
+    0,                                  /* tp_getset */
+    0, /* &PycairoSurface_Type, */      /* tp_base */
+    0,                                  /* tp_dict */
+    0,                                  /* tp_descr_get */
+    0,                                  /* tp_descr_set */
+    0,                                  /* tp_dictoffset */
+    0,                                  /* tp_init */
+    0,                                  /* tp_alloc */
+    (newfunc)svg_surface_new,           /* tp_new */
+    0,                                  /* tp_free */
+    0,                                  /* tp_is_gc */
+    0,                                  /* tp_bases */
+};
+#endif  /* CAIRO_HAS_SVG_SURFACE */
+
+
 /* Class Win32Surface(Surface) -------------------------------------------- */
 #if CAIRO_HAS_WIN32_SURFACE
 #include <cairo-win32.h>
@@ -752,26 +1035,14 @@ static PyObject *
 win32_surface_new (PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
     int hdc;
-    cairo_surface_t *surface;
-    PyObject *o;
 
-    if (!PyArg_ParseTuple(args, "i:Win32Surface.__new__",  &hdc))
+    if (!PyArg_ParseTuple(args, "i:Win32Surface.__new__", &hdc))
 	return NULL;
-
-    o = type->tp_alloc(type, 0);
-    if (o) {
-	surface = cairo_win32_surface_create ((HDC)hdc);
-	if (Pycairo_Check_Status (cairo_surface_status (surface))) {
-	    cairo_surface_destroy (surface);
-	    Py_DECREF(o);
-	    return NULL;
-	}
-	((PycairoPSSurface *)o)->surface = surface;
-    }
-    return o;
+    return PycairoSurface_FromSurface (
+	       cairo_win32_surface_create ((HDC)hdc), NULL);
 }
 
-static PyMethodDef win32surface_methods[] = {
+static PyMethodDef win32_surface_methods[] = {
     {NULL, NULL, 0, NULL},
 };
 
@@ -804,7 +1075,7 @@ PyTypeObject PycairoWin32Surface_Type = {
     0,                                  /* tp_weaklistoffset */
     0,                                  /* tp_iter */
     0,                                  /* tp_iternext */
-    win32surface_methods,               /* tp_methods */
+    win32_surface_methods,              /* tp_methods */
     0,                                  /* tp_members */
     0,                                  /* tp_getset */
     0, /* &PycairoSurface_Type, */      /* tp_base */
@@ -822,6 +1093,90 @@ PyTypeObject PycairoWin32Surface_Type = {
 #endif  /* CAIRO_HAS_WIN32_SURFACE */
 
 
+/* Class XlibSurface(Surface) --------------------------------------------- */
+#ifdef CAIRO_HAS_XLIB_SURFACE
+#include <cairo-xlib.h>
+
+static PyObject *
+xlib_surface_new (PyTypeObject *type, PyObject *args, PyObject *kwds)
+{
+    PyErr_SetString(PyExc_TypeError,
+		    "The XlibSurface type cannot be directly instantiated");
+    return NULL;
+}
+
+static PyObject *
+xlib_surface_get_depth (PycairoXlibSurface *o)
+{
+    return PyInt_FromLong (cairo_xlib_surface_get_depth (o->surface));
+}
+
+static PyObject *
+xlib_surface_get_height (PycairoXlibSurface *o)
+{
+    return PyInt_FromLong (cairo_xlib_surface_get_height (o->surface));
+}
+
+static PyObject *
+xlib_surface_get_width (PycairoXlibSurface *o)
+{
+    return PyInt_FromLong (cairo_xlib_surface_get_width (o->surface));
+}
+
+static PyMethodDef xlib_surface_methods[] = {
+    {"get_depth", (PyCFunction)xlib_surface_get_depth,    METH_NOARGS },
+    {"get_height",(PyCFunction)xlib_surface_get_height,   METH_NOARGS },
+    {"get_width", (PyCFunction)xlib_surface_get_width,    METH_NOARGS },
+    {NULL, NULL, 0, NULL},
+};
+
+PyTypeObject PycairoXlibSurface_Type = {
+    PyObject_HEAD_INIT(NULL)
+    0,                                  /* ob_size */
+    "cairo.XlibSurface",                /* tp_name */
+    sizeof(PycairoXlibSurface),         /* tp_basicsize */
+    0,                                  /* tp_itemsize */
+    0,                                  /* tp_dealloc */
+    0,                                  /* tp_print */
+    0,                                  /* tp_getattr */
+    0,                                  /* tp_setattr */
+    0,                                  /* tp_compare */
+    0,                                  /* tp_repr */
+    0,                                  /* tp_as_number */
+    0,                                  /* tp_as_sequence */
+    0,                                  /* tp_as_mapping */
+    0,                                  /* tp_hash */
+    0,                                  /* tp_call */
+    0,                                  /* tp_str */
+    0,                                  /* tp_getattro */
+    0,                                  /* tp_setattro */
+    0,                                  /* tp_as_buffer */
+    Py_TPFLAGS_DEFAULT,                 /* tp_flags */
+    0,                                  /* tp_doc */
+    0,                                  /* tp_traverse */
+    0,                                  /* tp_clear */
+    0,                                  /* tp_richcompare */
+    0,                                  /* tp_weaklistoffset */
+    0,                                  /* tp_iter */
+    0,                                  /* tp_iternext */
+    xlib_surface_methods,               /* tp_methods */
+    0,                                  /* tp_members */
+    0,                                  /* tp_getset */
+    0, /* &PycairoSurface_Type, */      /* tp_base */
+    0,                                  /* tp_dict */
+    0,                                  /* tp_descr_get */
+    0,                                  /* tp_descr_set */
+    0,                                  /* tp_dictoffset */
+    0,                                  /* tp_init */
+    0,                                  /* tp_alloc */
+    (newfunc)xlib_surface_new,          /* tp_new */
+    0,                                  /* tp_free */
+    0,                                  /* tp_is_gc */
+    0,                                  /* tp_bases */
+};
+#endif  /* CAIRO_HAS_XLIB_SURFACE */
+
+
 /* Numeric routines ------------------------------------------------------- */
 
 #ifdef HAVE_NUMPY
